@@ -28,9 +28,11 @@ const GtkChamplain = imports.gi.GtkChamplain;
 const Lang = imports.lang;
 
 const Application = imports.application;
+const ContactPlace = imports.contactPlace;
+const Geoclue = imports.geoclue;
 const MapWalker = imports.mapWalker;
 const Place = imports.place;
-const SearchResultMarker = imports.searchResultMarker;
+const PlaceMarker = imports.placeMarker;
 const TurnPointMarker = imports.turnPointMarker;
 const UserLocationMarker = imports.userLocationMarker;
 const Utils = imports.utils;
@@ -78,7 +80,9 @@ const MapView = new Lang.Class({
         this.setMapType(MapType.STREET);
 
         this._updateUserLocation();
-        Application.geoclue.connect("location-changed",
+        Application.geoclue.connect('location-changed',
+                                    this._updateUserLocation.bind(this));
+        Application.geoclue.connect('notify::state',
                                     this._updateUserLocation.bind(this));
 
         this._connectRouteSignals();
@@ -113,8 +117,8 @@ const MapView = new Lang.Class({
         this.view.add_layer(this._routeLayer);
 
         let mode = Champlain.SelectionMode.SINGLE;
-        this._searchResultLayer = new Champlain.MarkerLayer({ selection_mode: mode });
-        this.view.add_layer(this._searchResultLayer);
+        this._placeLayer = new Champlain.MarkerLayer({ selection_mode: mode });
+        this.view.add_layer(this._placeLayer);
 
         this._instructionMarkerLayer = new Champlain.MarkerLayer({ selection_mode: mode });
         this.view.add_layer(this._instructionMarkerLayer);
@@ -164,9 +168,16 @@ const MapView = new Lang.Class({
     },
 
     _updateUserLocation: function() {
-        if (!Application.geoclue || !Application.geoclue.place)
+        if (!Application.geoclue.place)
             return;
 
+        if (Application.geoclue.state !== Geoclue.State.ON) {
+            if (this._userLocation)
+                this._userLocation.destroy();
+            this._userLocation = null;
+            return;
+        }
+
         let place = Application.geoclue.place;
 
         let previousSelected = this._userLocation && this._userLocation.selected;
@@ -180,6 +191,19 @@ const MapView = new Lang.Class({
         this.emit('user-location-changed');
     },
 
+    _gotoBBox: function(bbox) {
+        let [lat, lon] = bbox.get_center();
+        let place = new Place.Place({
+            location: new Geocode.Location({ latitude  : lat,
+                                             longitude : lon }),
+            bounding_box: new Geocode.BoundingBox({ top    : bbox.top,
+                                                    bottom : bbox.bottom,
+                                                    left   : bbox.left,
+                                                    right  : bbox.right })
+        });
+        new MapWalker.MapWalker(place, this).goTo(true);
+    },
+
     showTurnPoint: function(turnPoint) {
         if (this._turnPointMarker)
             this._turnPointMarker.destroy();
@@ -190,38 +214,48 @@ const MapView = new Lang.Class({
         this._turnPointMarker = new TurnPointMarker.TurnPointMarker({ turnPoint: turnPoint,
                                                                       mapView: this });
         this._instructionMarkerLayer.add_marker(this._turnPointMarker);
-        this._turnPointMarker.goToAndSelect(true);
+        this._turnPointMarker.goTo();
+    },
+
+    showContact: function(contact) {
+        let places = contact.get_places();
+        if (places.length === 0)
+            return;
+
+        this._placeLayer.remove_all();
+        places.forEach((function(p) {
+            let place = new ContactPlace.ContactPlace({ place: p,
+                                                        contact: contact });
+            let marker = new PlaceMarker.PlaceMarker({ place: place,
+                                                       mapView: this });
+            this._placeLayer.add_marker(marker);
+        }).bind(this));
+
+        this._gotoBBox(contact.bounding_box);
     },
 
     showSearchResult: function(place) {
-        this._searchResultLayer.remove_all();
-        let searchResultMarker = new SearchResultMarker.SearchResultMarker({ place: place,
-                                                                             mapView: this });
+        this._placeLayer.remove_all();
+        this.routeVisible = false;
+        let placeMarker = new PlaceMarker.PlaceMarker({ place: place,
+                                                        mapView: this });
 
-        this._searchResultLayer.add_marker(searchResultMarker);
-        searchResultMarker.goToAndSelect(true);
+        this._placeLayer.add_marker(placeMarker);
+        placeMarker.goToAndSelect(true);
 
-        return searchResultMarker;
+        return placeMarker;
     },
 
     showRoute: function(route) {
         this._routeLayer.remove_all();
+        this._placeLayer.remove_all();
+
         this.routeVisible = true;
 
         route.path.forEach(this._routeLayer.add_node.bind(this._routeLayer));
 
-        let [lat, lon] = route.bbox.get_center();
-        let place = new Place.Place({
-            location     : new Geocode.Location({ latitude  : lat,
-                                                  longitude : lon }),
-            bounding_box : new Geocode.BoundingBox({ top    : route.bbox.top,
-                                                     bottom : route.bbox.bottom,
-                                                     left   : route.bbox.left,
-                                                     right  : route.bbox.right })
-        });
-
         this._showDestinationTurnpoints();
-        new MapWalker.MapWalker(place, this).goTo(true);
+        this._gotoBBox(route.bbox);
     },
 
     _showDestinationTurnpoints: function() {
@@ -233,9 +267,9 @@ const MapView = new Lang.Class({
         route.turnPoints.forEach(function(turnPoint) {
             if (turnPoint.isStop()) {
                 let queryPoint = query.filledPoints[pointIndex];
-                let destinationMarker = new TurnPointMarker.DestinationMarker({ turnPoint: turnPoint,
-                                                                                queryPoint: queryPoint,
-                                                                                mapView: this });
+                let destinationMarker = new TurnPointMarker.TurnPointMarker({ turnPoint: turnPoint,
+                                                                              queryPoint: queryPoint,
+                                                                              mapView: this });
                 this._instructionMarkerLayer.add_marker(destinationMarker);
                 pointIndex++;
             }
